/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

#define FE_SMO_DUMP		FALSE

#include "operator/operator.pmh"

namespace fe
{

void SurfaceMetricOp::initialize(void)
{
	catalog<String>("icon")="FE_alpha";

	catalog<String>("ValueAttr")="value";
	catalog<String>("ValueAttr","label")="Value Attribute";
	catalog<String>("ValueAttr","hint")=
			"Name of point attribute to place results.";

	catalog<Real>("Minimum")= -1.0;
	catalog<Real>("Minimum","low")= -10.0;
	catalog<Real>("Minimum","high")=0.0;

	catalog<Real>("Maximum")=1.0;
	catalog<Real>("Maximum","high")=10.0;

	catalog<I32>("Width")=256;
	catalog<I32>("Width","high")=1024;

	catalog<I32>("Height")=256;
	catalog<I32>("Height","high")=1024;

	catalog< sp<Component> >("Input Surface");

	catalog< sp<Component> >("Contrasting Surface");

	//* don't copy input to output
	catalog< sp<Component> >("Output Surface");
	catalog<String>("Output Surface","IO")="output";
}

void SurfaceMetricOp::handle(Record& a_rSignal)
{
	const String valueAttr=catalog<String>("ValueAttr");
	const Real minimum=catalog<Real>("Minimum");
	const Real maximum=catalog<Real>("Maximum");
	const I32 width=catalog<I32>("Width");
	const I32 height=catalog<I32>("Height");

	sp<DrawI> spDrawI;
	if(!accessDraw(spDrawI,a_rSignal)) return;

	sp<SurfaceAccessibleI> spOutputAccessible;
	if(!accessOutput(spOutputAccessible,a_rSignal)) return;

	sp<SurfaceAccessorI> spOutputUVs;
	if(!access(spOutputUVs,spOutputAccessible,e_point,"uv")) return;

	sp<SurfaceAccessorI> spOutputWidth;
	if(!access(spOutputWidth,spOutputAccessible,
			e_detail,"raster_width")) return;

	sp<SurfaceAccessorI> spOutputHeight;
	if(!access(spOutputHeight,spOutputAccessible,
			e_detail,"raster_height")) return;

	sp<SurfaceAccessorI> spOutputMin;
	if(!access(spOutputMin,spOutputAccessible,
			e_detail,"raster_min")) return;

	sp<SurfaceAccessorI> spOutputMax;
	if(!access(spOutputMax,spOutputAccessible,
			e_detail,"raster_max")) return;

	sp<SurfaceAccessorI> spOutputValue;
	if(!access(spOutputValue,spOutputAccessible,e_point,valueAttr)) return;

	sp<SurfaceAccessibleI> spInputAccessible;
	if(!access(spInputAccessible,"Input Surface")) return;

	sp<SurfaceI> spInput;
	if(!access(spInput,spInputAccessible)) return;

	sp<SurfaceI> spContrast;
	if(!access(spContrast,"Contrasting Surface")) return;

	spOutputWidth->set(0,width);
	spOutputHeight->set(0,height);
	spOutputMin->set(0,minimum);
	spOutputMax->set(0,maximum);

	const Real maxDistance=maximum-minimum;

	U32 pointCount=0;

	for(U32 vIndex=0;vIndex<height;vIndex++)
	{
		const Real vCoord=vIndex/(height-1.0);

		for(U32 uIndex=0;uIndex<width;uIndex++)
		{
			const Real uCoord=uIndex/(width-1.0);

			const SpatialVector uv(uCoord,vCoord);
			const SpatialTransform sample=spInput->sample(uv);
			const SpatialVector point=sample.translation();
			const SpatialVector normal=sample.column(1);

			const SpatialVector origin=point+minimum*normal;
			sp<SurfaceI::ImpactI> spImpact=
					spContrast->rayImpact(origin,normal,maxDistance);

			Real value=maximum;
			if(spImpact.isValid())
			{
				value=spImpact->distance()+minimum;
			}

			spDrawI->drawPoints(&point,NULL,1,false,NULL);

			spOutputUVs->set(pointCount,uv);
			spOutputValue->set(pointCount,value);

			pointCount++;

#if FE_SMO_DUMP
			feLog("%3d ",I32(value*100));
#endif
		}
#if FE_SMO_DUMP
		feLog("\n");
#endif
	}
}

} // namespace
