/*	Copyright (C) 2003-2010 Free Electron Organization
	Any use of this software requires a license.  If a valid license
	was not distributed with this file, visit freeelectron.org. */

/** @file */

#ifndef __shape_shapeAS_h__
#define __shape_shapeAS_h__

namespace fe
{

class AsPoint :
	virtual public AccessorSet,
	public Initialize<AsPoint>
{
	public:
		void initialize(void)
		{
			add(location,	FE_SPEC("spc:at",		"point location"));
		}
		/// location
		Accessor<SpatialVector>					location;
};

class AsVelocityPoint :
	virtual public AsPoint,
	public Initialize<AsVelocityPoint>
{
	public:
		void initialize(void)
		{
			add(velocity,			FE_USE("spc:velocity"));
		}
		/// spatial velocity
		Accessor<SpatialVector>		velocity;
};

/// force application point
class AsForcePoint :
	public AsVelocityPoint,
	public Initialize<AsForcePoint>
{
	public:
		void initialize(void)
		{
			add(force,				FE_USE("sim:force"));
		}

		/// spatial force
		Accessor<SpatialVector>		force;
};


/// particle in physical space
class AsParticle :
	public AsForcePoint,
	public Initialize<AsParticle>
{
	public:
		void initialize(void)
		{
			add(mass,				FE_USE("sim:mass"));
		}
		/// mass
		Accessor<Real>				mass;
};

/// possible bounding area/volume attributes
class AsBounded :
	virtual public AccessorSet,
	public Initialize<AsBounded>
{
	public:
		void initialize(void)
		{
			add(radius,				FE_USE("bnd:radius"));
		}
		/// spatial radius
		Accessor<Real>				radius;
};

class AsTransform :
	virtual public AccessorSet,
	public Initialize<AsTransform>
{
	public:
		void initialize(void)
		{
			add(transform,			FE_USE("spc:transform"));
		}
		/// transform
		Accessor<SpatialTransform>	transform;
};

class FE_DL_EXPORT AsColored:
	public AccessorSet,
	public Initialize<AsColored>
{
	public:
		void initialize(void)
		{
			add(color,			FE_USE("sim:color"));
		}
		Accessor<Color>			color;
};

/// Labeled
class AsLabeled :
	virtual public AccessorSet,
	public Initialize<AsLabeled>
{
	public:
		void initialize(void)
		{
			add(label,				FE_USE("sim:label"));
		}
		/// label
		Accessor<String>			label;
};


} /* namespace */

#endif /* __shape_shapeAS_h__ */

